
#include "bda/CollapsibleDockWidget.hh"

#include <QLayout>
#include <QStyleOption>
#include <QPushButton>

#include <iostream>

CollapsibleDockWidget::CollapsibleDockWidget(const QString& title, QWidget* parent, Qt::WindowFlags flags)
    : QDockWidget(title, parent, flags) {
    mCollapsed = false;

    vButton = new QPushButton("", this);
    setStyleSheet("QDockWidget::title { text-align: left; background : red; padding-left: 16px; }");
    if (features() & QDockWidget::DockWidgetVerticalTitleBar) {
        vButton->setIcon(this->style()->standardIcon(QStyle::SP_ToolBarHorizontalExtensionButton));
        vButton->setGeometry(3, this->height() - 17, 14, 14);
    } else {
        vButton->setIcon(this->style()->standardIcon(QStyle::SP_ToolBarVerticalExtensionButton));
        vButton->setGeometry(3, 3, 14, 14);
    }
    QObject::connect(vButton, &QAbstractButton::clicked, this, &CollapsibleDockWidget::onCollapseToggle);
}

CollapsibleDockWidget::CollapsibleDockWidget(QWidget* parent, Qt::WindowFlags flags)
    : CollapsibleDockWidget(tr(""), parent, flags) {
}

CollapsibleDockWidget::~CollapsibleDockWidget() {
    mWidget->deleteLater();
}

void CollapsibleDockWidget::setWidget(QWidget* widget) {
    mWidget = widget;
    if (mCollapsed) {
        QDockWidget::setWidget(nullptr);
    } else {
        QDockWidget::setWidget(mWidget);
    }
}

void CollapsibleDockWidget::collapse(const bool aCollapse) {
    mCollapsed = aCollapse;
    if (mCollapsed) {
        mOriginalMinHeight = this->minimumHeight();
        mOriginalMaxHeight = this->maximumHeight();
        mOriginalHeight = this->height();

        this->setMinimumHeight(5);
        this->setMaximumHeight(20);
        this->resize(this->width(), 5);
        vButton->setIcon(this->style()->standardIcon(QStyle::SP_ToolBarHorizontalExtensionButton));
    } else {
        this->setMinimumHeight(mOriginalMinHeight);
        this->setMaximumHeight(mOriginalMaxHeight);
        this->resize(this->width(),  mOriginalHeight);
        vButton->setIcon(this->style()->standardIcon(QStyle::SP_ToolBarVerticalExtensionButton));
    }
    setWidget(mWidget);
    emit collapsed(mCollapsed);
}

void CollapsibleDockWidget::collapsed(const bool aCollapsed) {
}

bool CollapsibleDockWidget::isCollapsed() {
    return mCollapsed;
}

void CollapsibleDockWidget::onCollapseToggle() {
    collapse(!mCollapsed);
}
